unit n2_mathe;

interface

function inf:double;
function nan:double;
function isnan(x:double):boolean;
function gauss(x:double):double;
function erf(x:double):double;
function fac(n:longint):double;
function gamma(x:double):double;
function lngamma(x:double):double;
function chi_alpha(alpha:double;n:longint):double;
function t_alpha(alpha:double;n:longint):double;
function sgn(x:double):longint;
function ceil(x:double):longint;
function floor(x:double):longint;
function round(x:double):longint;
function maxd(x1,x2:double):double;
function mind(x1,x2:double):double;
function maxi(x1,x2:longint):longint;
function mini(x1,x2:longint):longint;
function ln(x:double):double;
function lg(x:double):double;
function ld(x:double):double;
function exp(x:double):double;
function exp10(x:double):double;
function exp2(x:double):double;
function log(a,b:double):double;
function power(a,e:double):double;
function si(x:double):double;
function tan(x:double):double;
function cot(x:double):double;
function sec(x:double):double;
function csc(x:double):double;
function arcsin(x:double):double;
function arccos(x:double):double;
function arccot(x:double):double;
function arcsec(x:double):double;
function arccsc(x:double):double;
function sinh(x:double):double;
function cosh(x:double):double;
function tanh(x:double):double;
function coth(x:double):double;
function sech(x:double):double;
function csch(x:double):double;
function arsinh(x:double):double;
function arcosh(x:double):double;
function artanh(x:double):double;
function arcoth(x:double):double;
function arsech(x:double):double;
function arcsch(x:double):double;

implementation

uses math;

function inf:double;
begin
  inf:=infinity;
end;

function nan:double;
begin
  nan:=math.nan;
end;

function isnan(x:double):boolean;
begin
  isnan:=math.isnan(x);
end;

function gauss(x:double):double;
begin
  if abs(x)<100 then gauss:=exp(-sqr(x)/2)/sqrt(2*pi) else gauss:=0;
end;

function erf(x:double):double;
var s,f,fo,x100:double;
    i:longint;
begin
  if x<0 then erf:=-erf(-x)
  else
  begin
    if x<10 then
    begin
      s:=0;
      fo:=0;
      x100:=100*x;
      for i:=0 to trunc(x100+1) do
      begin
        f:=exp(-sqr(i/100));
        if i>0 then s:=s+(f+fo)*mind(1,x100);
        fo:=f;
      end;
      erf:=s/sqrt(pi)/100;
    end
    else erf:=1;
  end;
end;

function fac(n:longint):double;
var i:longint;
    f:double;
begin
  f:=1;
  for i:=1 to n do f:=f*i;
  fac:=f;
end;

function gamma(x:double):double;
var f,fi,fo,fn:double;
    t,dt:double;
begin
  if x=round(x) then gamma:=fac(round(x-1))
  else
  begin
    fi:=x;
    while fi<100 do fi:=fi+1;
    f:=0;
    t:=0;
    dt:=10/3;
    t:=dt/2;
    repeat
      fo:=f;
      f:=f+exp((fi-1)*ln(t)-t)*dt;
      t:=t+dt;
      fn:=f;
    until fo=fn;
    while fi>x+0.5 do
    begin
      fi:=fi-1;
      f:=f/fi;
    end;
    gamma:=f;
  end;
end;

function lngamma(x:double):double;
var i:longint;
    s:double;
begin
  if frac(x)>0 then
  begin
    s:=ln(gamma(frac(x)));
    for i:=0 to trunc(x)-1 do s:=s+ln(frac(x)+i);
  end
  else
  begin
    s:=0;
    for i:=1 to trunc(x)-1 do s:=s+ln(i);
  end;
  lngamma:=s;
end;

function chi_alpha(alpha:double;n:longint):double;
var v:double;
    i:longint;
    s,p,q:double;
    l1,l2,s1,x1,x2,y1,y2:double;
begin
  v:=-n/2*ln(2)-lngamma(n/2);
  i:=0;
  s:=0;
  while s<1-alpha do
  begin
    x1:=i/1000*n;
    x2:=(i+1)/1000*n;
    if x1>0 then y1:=exp((n/2-1)*ln(x1)-x1/2+v)
    else
    begin
      if n<2 then y1:=inf;
      if n=2 then y1:=0.5;
      if n>2 then y1:=0;
    end;
    y2:=exp((n/2-1)*ln(x2)-x2/2+v);
    s1:=s;
    s:=s+(y1+y2)/2*(x2-x1);
    inc(i);
  end;
  p:=(y1-(y2-y1)/(x2-x1)*x1)/((y2-y1)/(x2-x1)/2);
  q:=(s1-x1*(y1-(y2-y1)/(x2-x1)*x1)-(y2-y1)/(x2-x1)/2*sqr(x1)-(1-alpha))/((y2-y1)/(x2-x1)/2);
  l1:=-p/2-sqrt(sqr(p)/4-q);
  l2:=-p/2+sqrt(sqr(p)/4-q);
  if (l1>=x1) and (l1<=x2) then chi_alpha:=l1
  else
  begin
    if (l2>=x1) and (l2<=x2) then chi_alpha:=l2
    else writeln('error');
  end;
end;


function t_alpha(alpha:double;n:longint):double;
var v:double;
    i:longint;
    s,p,q:double;
    l1,l2,s1,x1,x2,y1,y2:double;
begin
  v:=-ln(sqrt(n*pi))-lngamma(n/2)+lngamma((n+1)/2);
  i:=0;
  s:=0;
  while s<(1-alpha)/2 do
  begin
    x1:=i/1000*n;
    x2:=(i+1)/1000*n;
    y1:=exp(-(n+1)/2*ln(1+sqr(x1)/n)+v);
    y2:=exp(-(n+1)/2*ln(1+sqr(x2)/n)+v);
    s1:=s;
    s:=s+(y1+y2)/2*(x2-x1);
	inc(i);
  end;
  p:=(y1-(y2-y1)/(x2-x1)*x1)/((y2-y1)/(x2-x1)/2);
  q:=(s1-x1*(y1-(y2-y1)/(x2-x1)*x1)-(y2-y1)/(x2-x1)/2*sqr(x1)-(1-alpha)/2)/((y2-y1)/(x2-x1)/2);
  l1:=-p/2-sqrt(sqr(p)/4-q);
  l2:=-p/2+sqrt(sqr(p)/4-q);
  if (l1>=x1) and (l1<=x2) then t_alpha:=l1
  else
  begin
    if (l2>=x1) and (l2<=x2) then t_alpha:=l2
    else writeln('error');
  end;
end;

function sgn(x:double):longint;
begin
  if x<0 then sgn:=-1 else sgn:=byte(x>0);
end;

function round(x:double):longint;
begin
  round:=system.trunc(x+sgn(x)*0.5);
end;

function ceil(x:double):longint;
begin
  ceil:=round(x-frac(x-trunc(x)-1));
end;

function floor(x:double):longint;
begin
  floor:=round(x-frac(x-trunc(x)+1));
end;

function maxd(x1,x2:double):double;
begin
  if x2>x1 then maxd:=x2 else maxd:=x1;
//  maxd:=(x1+x2+abs(x1-x2))/2;
end;

function mind(x1,x2:double):double;
begin
  if x2<x1 then mind:=x2 else mind:=x1;
//  mind:=(x1+x2-abs(x1-x2))/2;
end;

function maxi(x1,x2:longint):longint;
begin
  if x2>x1 then maxi:=x2 else maxi:=x1;
//  maxi:=round((x1+x2+abs(x1-x2))/2);
end;

function mini(x1,x2:longint):longint;
begin
  if x2<x1 then mini:=x2 else mini:=x1;
//  mini:=round((x1+x2-abs(x1-x2))/2);
end;

function ln(x:double):double;
begin
  if x<=1e308 then
  begin
//    if x>=1e-323 then ln:=system.ln(x) else ln:=-inf;
    if x>=0 then ln:=system.ln(x) else ln:=-inf;
  end
  else ln:=inf;
end;

function lg(x:double):double;
begin
   lg:=ln(x)/ln(10);
end;

function ld(x:double):double;
begin
   ld:=ln(x)/ln(2);
end;

function exp(x:double):double;
begin
  if x<=709 then
  begin
    if x>=-745 then exp:=system.exp(x) else exp:=0;
  end
  else exp:=inf;
end;

function exp10(x:double):double;
begin
  exp10:=exp(x*ln(10));
end;

function exp2(x:double):double;
begin
  exp2:=exp(x*ln(2));
end;

function log(a,b:double):double;
begin
  log:=ln(a)/ln(b);
end;

function power(a,e:double):double;
begin
  if a>0 then power:=exp(e*ln(a)) else
  begin if a=0 then power:=0 else power:=nan; end;
end;

function si(x:double):double;
begin
  if x<>0 then si:=sin(x)/x else si:=1;
end;

function tan(x:double):double;
begin
  if cos(x)<>0 then tan:=sin(x)/cos(x) else tan:=nan;
end;

function cot(x:double):double;
begin
  if sin(x)<>0 then cot:=cos(x)/sin(x) else cot:=nan;
end;

function sec(x:double):double;
begin
  if cos(x)<>0 then sec:=1/cos(x) else sec:=nan;
end;

function csc(x:double):double;
begin
  if sin(x)<>0 then csc:=1/sin(x) else csc:=nan;
end;

function arcsin(x:double):double;
begin
  if sqr(x)<1 then arcsin:=arctan(x/sqrt(1-sqr(x))) else
  begin
    if sqr(x)=1 then arcsin:=x*pi/2 else arcsin:=nan;
  end;
end;

function arccos(x:double):double;
begin
  if sqr(x)<1 then arccos:=pi/2-arctan(x/sqrt(1-sqr(x))) else
  begin
    if sqr(x)=1 then arccos:=(1-x)*pi/2 else arccos:=nan;
  end;
end;

function arccot(x:double):double;
begin
  arccot:=pi/2-arctan(x);
end;

function arcsec(x:double):double;
begin
  if sqr(x)>=1 then arcsec:=pi*byte(x<0)+sgn(x)*arctan(sqrt(sqr(x)-1)) else arcsec:=nan;
end;

function arccsc(x:double):double;
begin
  if sqr(x)>1 then arccsc:=sgn(x)*arctan(1/sqrt(sqr(x)-1)) else
  begin
    if sqr(x)=1 then arccsc:=x*pi/2 else arccsc:=nan;
  end;
end;

function sinh(x:double):double;
begin
  sinh:=(exp(x)-exp(-x))/2;
end;

function cosh(x:double):double;
begin
  cosh:=(exp(x)+exp(-x))/2;
end;

function tanh(x:double):double;
begin
  tanh:=sinh(x)/cosh(x);
end;

function coth(x:double):double;
begin
  if sinh(x)<>0 then coth:=cosh(x)/sinh(x) else coth:=nan;
end;

function sech(x:double):double;
begin
  sech:=1/cosh(x);
end;

function csch(x:double):double;
begin
  if sinh(x)<>0 then csch:=1/sinh(x) else csch:=nan;
end;

function arsinh(x:double):double;
begin
  arsinh:=ln(x+sqrt(sqr(x)+1));
end;

function arcosh(x:double):double;
begin
  if x>=1 then arcosh:=ln(x+sqrt(sqr(x)-1)) else arcosh:=nan;
end;

function artanh(x:double):double;
begin
  if abs(x)<1 then artanh:=ln((1+x)/(1-x))/2 else artanh:=nan;
end;

function arcoth(x:double):double;
begin
  if abs(x)>1 then arcoth:=ln((x+1)/(x-1))/2 else arcoth:=nan;
end;

function arsech(x:double):double;
begin
  if (x>0) and (x<=1) then arsech:=ln((sqrt(1-sqr(x))+1)/x) else arsech:=nan;
end;

function arcsch(x:double):double;
begin
  if x<>0 then arcsch:=ln((sqrt(1+sqr(x))+1)/abs(x))*sgn(x) else arcsch:=nan;
end;

end.
